import os
import pytest
import serial
import stat
import time

@pytest.fixture
def firmware_file_serial(trtl_firmware_dir):
    return os.path.join(trtl_firmware_dir, "firmware/serial/fw-serial.bin")

@pytest.fixture
def firmware_output():
    return ["this is a message\r\n",
                ]

    # FIXME test different print level
    # return ["this is a message\r\n",
    #         "this is a message\r\nand we use it for testing\r\n",
    #         "this is a message\r\nthat goes through the serial interface\n\rand we use it for testing\r\n"]


class TestSerial(object):
    def test_exist(self, cpu_idx):
        assert os.path.exists("/dev/ttyTRTL{:d}".format(cpu_idx)) == True

    def test_chardevice(self, cpu_idx):
        st = os.stat("/dev/ttyTRTL{:d}".format(cpu_idx))
        assert stat.S_ISCHR(st.st_mode) == True

    def test_permission(self, cpu_idx):
        st = os.stat("/dev/ttyTRTL{:d}".format(cpu_idx))
        assert st.st_mode & stat.S_IRUSR != 0
        assert st.st_mode & stat.S_IRGRP != 0

    @pytest.mark.parametrize("msg_ref", enumerate(firmware_output()))
    def test_print(self, trtl_cpu, msg_ref, firmware_file_serial):
        trtl_cpu.disable()
        trtl_cpu.load_application_file(firmware_file_serial)

        with serial.Serial("/dev/ttyTRTL{:d}".format(trtl_cpu.idx_cpu)) as ser:
            trtl_cpu.enable()
            time.sleep(1)  # wait for all the characters

            nbyte = len(msg_ref[1])
            msg = ser.read(nbyte).decode()

            assert msg_ref[1] == msg
